using ColorTypes
import Base.show
import Base.copy
import Base.setindex!


struct Cell
	char:: Char
	foreground_color:: Union{Color, Nothing}
	background_color:: Union{Color, Nothing}
	
	Cell(char:: Char, foreground_color:: Union{Color, Nothing}, background_color:: Union{Color, Nothing}) = new(char, foreground_color, background_color)
	Cell(char:: Char) = new(char, nothing, nothing)
end

function show(io:: IO, cell:: Cell, reset_style_after:: Bool = true)
	set_foreground_color(io, cell.foreground_color)
	set_background_color(io, cell.background_color)
	print(io, cell.char)
	reset_style_after && reset_style(io)
end
show(cell:: Cell, reset_style_after:: Bool = true) = show(stdout, cell, reset_style_after)


mutable struct Buffer
	buffer_pos:: Tuple{<:Integer, <:Integer}
	cells:: Matrix{Cell}
	cursor_pos:: Tuple{<:Integer, <:Integer}
end

copy(buffer:: Buffer) = Buffer(buffer.buffer_pos, copy(buffer.cells), buffer.cursor_pos)

function draw_buffer(io:: IO, buffer:: Buffer, reset_style_after:: Bool = true)
	move_cursor(buffer.buffer_pos...)
	for j in 1:size(buffer.cells, 2)
		for i in 1:size(buffer.cells, 1)
			show(io, buffer.cells[i, j], false)
		end
		move_cursor(buffer.buffer_pos[1], buffer.buffer_pos[2] + j)
	end
	reset_style_after && reset_style(io)
	move_cursor(buffer.cursor_pos...)
end
draw_buffer(buffer:: Buffer, reset_style_after:: Bool = true) = draw_buffer(stdout, buffer, reset_style_after)

getindex(buffer:: Buffer, x:: Integer, y:: Integer) = getindex(buffer.cells, x, y)
setindex!(buffer:: Buffer, cell:: Cell, x:: Integer, y:: Integer) = setindex!(buffer.cells, cell, x, y)


CURRENT_BUFFER = Buffer(
	(1, 1),
	fill(Cell(' '), displaysize(stdout)[2], displaysize(stdout)[1]),
	(1, 1)
)

WORKING_BUFFER = Buffer(
	(1, 1),
	fill(Cell(' '), displaysize(stdout)[2], displaysize(stdout)[1]),
	(1, 1)
)


include("display_management.jl")
