function clear_screen(io:: IO = stdout)
	print(io, "\e[J")
end

function move_cursor(io:: IO, x:: Integer, y:: Integer)
	print(io, "\e[$y;$(x)H") # or f
end
move_cursor(x:: Integer, y:: Integer) = move_cursor(stdout, x, y)

function move_cursor_home(io:: IO = stdout)
	print(io, "\e[H")
end

function move_cursor_up(io:: IO, x:: Integer = 1)
	print(io, "\e[$(x)A")
end
move_cursor_up(x:: Integer = 1) = move_cursor_up(stdout, x)

function move_cursor_down(io:: IO, x:: Integer = 1)
	print(io, "\e[$(x)B")
end
move_cursor_down(x:: Integer = 1) = move_cursor_down(stdout, x)

function move_cursor_right(io:: IO, x:: Integer = 1)
	print(io, "\e[$(x)C")
end
move_cursor_right(x:: Integer = 1) = move_cursor_right(stdout, x)

function move_cursor_left(io:: IO, x:: Integer = 1)
	print(io, "\e[$(x)D")
end
move_cursor_left(x:: Integer = 1) = move_cursor_left(stdout, x)

function erase_line(io:: IO = stdout)
	print(io, "\e[K")
end

function save_screen(io:: IO = stdout)
	print(io, "\e[?47h")
end

function restore_screen(io:: IO = stdout)
	print(io, "\e[?47l")
end

function show_cursor(io:: IO = stdout)
	print(io, "\e[?25h")
end

function hide_cursor(io:: IO = stdout)
	print(io, "\e[?25l")
end

function reset_style(io:: IO = stdout)
	print(io, "\e[0m")
end

function reset_foreground_color(io:: IO)
	print(io, "\e[39m")
end
reset_foreground_color() = reset_foreground_color(stdout)

function reset_background_color(io:: IO)
	print(io, "\e[49m")
end
reset_background_color() = reset_background_color(stdout)

function set_foreground_color(io:: IO, color:: Union{Color, Nothing})
	if color == nothing
		print(io, "\e[39m")
	else
		r = Int(round(red(color) * 255))
		g = Int(round(green(color) * 255))
		b = Int(round(blue(color) * 255))
		print(io, "\e[38;2;$r;$g;$(b)m")
	end
end
set_foreground_color(color:: Union{Color, Nothing}) = set_foreground_color(stdout, color)

function set_background_color(io:: IO, color:: Union{Color, Nothing})
	if color == nothing
		print(io, "\e[49m")
	else
		r = Int(round(red(color) * 255))
		g = Int(round(green(color) * 255))
		b = Int(round(blue(color) * 255))
		print(io, "\e[48;2;$r;$g;$(b)m")
	end
end
set_background_color(color:: Union{Color, Nothing}) = set_background_color(stdout, color)
